
 /*-------------------------------------------------------------------------*/
 /*---Schabenberger, O. and Gotway, C. A. (2005), "Statistical Methods for--*/
 /*---Spatial Data Analysis", Boca Raton, FL, CRC Press, Chapman & Hall  ---*/
 /*-------------------------------------------------------------------------*/

 /*-------------------------------------------------*/
 /*---Table 4.2, page 176                        ---*/
 /*-------------------------------------------------*/

%include "&progdir\CNRatioData.sas";

 /*-------------------------------------------------*/
/* %CL: A composite likelihood macro for 
        semivariogram estimation
   Macro arguments
   data   =    A data set of squared differences produced by 
               proc variogram
    maxrng=    The largest lag distance to be used in estimation
    minrng=    The lower bound on the range parameter estimate
               If no value is given no lower bound is enforced
    nugget=    1: include nugget effect
               0: fit no-nugget model
    covmod=    The spatial covariance model.
               E: Exponential, S: Spherical, G: Gaussian
    nuggetstart=, sillstart=, rangestart=
               Starting values for the model parameters           
*/
 /*-------------------------------------------------*/


%macro cl(data        = pairs,
          maxrng      = 200  ,
          minrng      =      ,
          nugget      = 1    ,
          covmod      = E    ,
          nuggetstart = 0.005,
          sillstart   = 0.03 ,
          rangestart  = 80   ,
          weight      = 1    ,
          outest      = est  );
  ods select ParameterEstimates;
  proc nlin data=&data(where=(distance <= &maxrng)) nohalve;
    %if &nugget %then %do;
      parameters Nugget=&nuggetstart Sill=&sillstart Range=&rangestart;
      %if %length(&minrng) %then %do;
                       bounds Nugget > 0, Sill > 0, Range > &minrng;
      %end; %else %do; bounds Nugget > 0, Sill > 0, Range > 0; %end;
    %end; %else %do;
      parameters Sill=&sillstart Range=&rangestart;
      Nugget = 0;
      %if %length(&minrng) %then %do;   bounds Sill > 0, Range > &minrng;
      %end; %else %do;                  bounds Sill > 0, Range > 0; %end;
    %end;
    %if (&Covmod=E) %then %do;
      semivariogram = Nugget + (Sill-Nugget)*(1-exp(-3*distance/Range));
    %end; %else %if (&Covmod=G) %then %do;
      semivariogram = Nugget + (Sill-Nugget)*(1-exp(-3*(distance/Range)**2));
    %end; %else %if (&Covmod=S) %then %do;
      semivariogram = Nugget + (Sill-Nugget)*(1.5*distance/Range - 
                                              0.5*(distance/Range)**3);
    %end;
    tij = (v1-v2)**2;
    %if (&weight=1) %then %do; 
	  _weight_ = 1/(8*(semivariogram**2));
    %end;
    model tij = 2*semivariogram;
    ods output ParameterEstimates=&outest;
  run;
  %if (&nugget=0) %then %do;
     data &outest; 
       set &outest end=last;
       if last then do;
          output;
          Parameter='Nugget';
	      Estimate = 0;
	      output;
       end; else output;
       keep Parameter Estimate;
     run;
  %end;
  proc transpose data=&outest out=&outest; 
     var Estimate; 
     id Parameter;
  run;
  data &outest; set &outest;
    length Method $4;
    %if (&weight=1) %then %do; 
	   Method='CL';
	%end; %else %do;
	   Method='GEE';
	%end;
    PartialSill = Sill - Nugget;
  run;
%mend cl;

 /*---Create a data set containing all distance pairs---*/
proc variogram data=cnratio outpair=pairs;
  compute novariogram;
  coordinates xcoord=x ycoord=y;
  var cn;
run;
proc means data=pairs min max mean; var distance; run;

title1 'Distribution of Distances for C/N Ratio Data';
proc gchart data=pairs;
  vbar distance;
run; quit;
title;


 /*-------------------------------------------------------*/
 /*-- Empirical semivariogram and WLS/OLS fit          ---*/
proc variogram data=cnratio outvar=svar(where=(distance ne .));
  compute lagdistance=6 maxlag=35;
  coordinates xcoord=x ycoord=y;
  var cn;
run;
proc print data=svar; run;

 /*--- OLS - no nugget                                 ---*/
proc nlin data=svar;
  parameters Sill=0.28 Range=50;
  semivariogram = Sill*(1-exp(-3*distance/Range));
  model variog = semivariogram;
  ods select ParameterEstimates;
  ods output ParameterEstimates=ols_nonugget;
run;
data ols_nonugget; 
  set ols_nonugget end=last;
  if last then do;
     output;
     Parameter='Nugget';
	 Estimate = 0;
	 output;
  end; else output;
  keep Parameter Estimate;
run;
proc transpose data=ols_nonugget out=ols_nonugget; 
   var Estimate; 
   id Parameter;
run;
data ols_nonugget; set ols_nonugget;
  length Method $4;
  Method = 'OLS';
  PartialSill = Sill - Nugget;
run;

 /*--- OLS - with nugget                               ---*/
proc nlin data=svar;
  parameters Nugget = 0.01 Sill=0.28 Range=50;
  bounds nugget > 0;
  semivariogram = Nugget + (Sill-Nugget)*(1-exp(-3*distance/Range));
  model variog = semivariogram;
  ods select ParameterEstimates;
  ods output ParameterEstimates=ols_nugget(keep=Parameter Estimate);
run;
proc transpose data=ols_nugget out=ols_nugget; 
   var Estimate; 
   id Parameter;
run;
data ols_nugget; set ols_nugget;
  length Method $4;
  Method = 'OLS';
  PartialSill = Sill - Nugget;
run;
  /*-------------------------------------------------------*/


  /*-------------------------------------------------------*/
  /*--- WLS - no nugget                                 ---*/
proc nlin data=svar nohalve;
  parameters Sill=0.28 Range=50;
  semivariogram = Sill*(1-exp(-3*distance/Range));
  _weight_ = .5*count/(semivariogram**2);
  model variog = semivariogram;
  ods select ParameterEstimates;
  ods output ParameterEstimates=wls_nonugget(keep=parameter Estimate);
run;
data wls_nonugget; 
  set wls_nonugget end=last;
  if last then do;
     output;
     Parameter='Nugget';
	 Estimate = 0;
	 output;
  end; else output;
  keep Parameter Estimate;
run;
proc transpose data=wls_nonugget out=wls_nonugget; 
   var Estimate; 
   id Parameter;
run;
data wls_nonugget; set wls_nonugget;
  length Method $4;
  Method = 'WLS';
  PartialSill = Sill - Nugget;
run;

 /*--- WLS - nugget                                     ---*/
proc nlin data=svar nohalve;
  parameters Nugget=0.01 Sill=0.28 Range=50;
  bounds Nugget > 0;
  semivariogram = Nugget + (Sill-Nugget)*(1-exp(-3*distance/Range));
  _weight_ = .5*count/(semivariogram**2);
  model variog = semivariogram;
  ods select ParameterEstimates;
  ods output ParameterEstimates=wls_nugget(keep=parameter Estimate);
run;
proc transpose data=wls_nugget out=wls_nugget; 
   var Estimate; 
   id Parameter;
run;
data wls_nugget; set wls_nugget;
  length Method $4;
  Method = 'WLS';
  PartialSill = Sill - Nugget;
run;
 /*-------------------------------------------------------*/


 /*-------------------------------------------------------*/
 /*---Composite likelihood estimates                   ---*/
 /*---No nugget, exponential                           ---*/
 %cl(nugget     = 0   , 
     covmod     = E   ,
     maxrng     = 200 ,
     sillstart  = 0.03,
     rangestart = 50  ,
     weight     = 1   ,
     outest     = cl_nonugget);

 /*---Nugget, exponential                              ---*/
 %cl(nugget      = 1    ,
     covmod      = E    ,
     maxrng      = 200  ,
     nuggetstart = 0.001,
     sillstart   = 0.03 ,
     rangestart  = 50   ,
     weight      = 1    ,    
     outest     = cl_nugget);
 /*-------------------------------------------------------*/


 /*-------------------------------------------------------*/
 /*---Generalized Estimating Equations                 ---*/
 /*---No nugget, exponential                           ---*/
 %cl(nugget     = 0   ,
     covmod     = E   ,
     maxrng     = 200 ,
     sillstart  = 0.03,
     rangestart = 50  ,
     weight     =  0  ,
     outest     = gee_nonugget);

 /*---Nugget, exponential                              ---*/
 %cl(nugget      = 1    ,
     covmod      = E    ,
     maxrng      = 200  ,
     nuggetstart = 0.001,
     sillstart   = 0.03 ,
     rangestart  = 50   ,
     weight      = 0    ,
     outest      = gee_nugget);
 /*-------------------------------------------------------*/


 /*-------------------------------------------------------*/
 /*---REML no-nugget                                   ---*/ 
 /*---NOTE: need to multiply the SP(EXP) estimate by 3 ---*/
 /*---      to get the practical range                 ---*/
proc mixed data=cnratio;
  model cn = / s;
  repeated / subject=intercept type=sp(exp)(x y);
  ods select ConvergenceStatus CovParms;
  ods output CovParms=Reml_nonugget;
run;
data Reml_nonugget;
  length Parameter $12;
  set Reml_nonugget;
  if (CovParm='SP(EXP)') then do;
     Parameter='Range';
	 Estimate = 3*Estimate;
  end;
  if (CovParm='Residual') then Parameter='Sill';
  keep Parameter Estimate;
run;
proc transpose data=Reml_nonugget out=Reml_nonugget; 
   var Estimate; 
   id Parameter;
run;
data Reml_nonugget; set Reml_nonugget;
  length Method $4;
  Method = 'REML';
  Nugget = 0;
  PartialSill = Sill - Nugget;
run;

/*---REML nugget                                      ---*/
proc mixed data=cnratio;
  model cn = / s;
  repeated / subject=intercept type=sp(exp)(x y) local;
  ods select ConvergenceStatus CovParms;
  ods output CovParms=Reml_nugget;
run;
data Reml_nugget;
  length Parameter $12;
  set Reml_nugget;
  if (CovParm='SP(EXP)') then do;
     Parameter='Range';
	 Estimate = 3*Estimate;
  end;
  if (CovParm='Residual') then Parameter='Nugget';
  if (CovParm='Variance') then Parameter='PartialSill';
  keep Parameter Estimate;
run;
proc transpose data=Reml_nugget out=Reml_nugget; 
   var Estimate; 
   id Parameter;
run;
data Reml_nugget; set Reml_nugget;
  length Method $4;
  Method = 'REML';
run;
 /*-------------------------------------------------------*/

 /*-------------------------------------------------------*/
 /*---Assemble the results as in Table 4.2             ---*/
 /*-------------------------------------------------------*/
data table_4_2;
  set ols_nonugget
      ols_nugget
	  wls_nonugget
	  wls_nugget
	  cl_nonugget
	  cl_nugget
	  gee_nonugget
	  gee_nugget
	  reml_nonugget
	  reml_nugget;
run;
proc print data=table_4_2 noobs label;
   label PartialSill = '(Partial) Sill'
         Range       = 'Practical Range';
   format Nugget best5. PartialSill 6.3 Range 6.2;
   var Method Nugget PartialSill Range;
run;

 /*-------------------------------------------------------*/
 /* or, use PROC GLIMMIX for REML*/
proc glimmix data=cnratio;
  model cn = / s;
  random _residual_ / subject=intercept type=sp(exp)(x y);
  ods select ConvergenceStatus CovParms;
run;

 /*---REML nugget                                      ---*/
proc glimmix data=cnratio;
  model cn = / s;
  random _residual_ / subject=intercept type=sp(exp)(x y);
  random _residual_; /* the nugget effect */
  ods select ConvergenceStatus CovParms;
run;
 /*-------------------------------------------------------*/


