 /*-------------------------------------------------------------------------*/
 /*---Schabenberger, O. and Gotway, C. A. (2005), "Statistical Methods for--*/
 /*---Spatial Data Analysis", Boca Raton, FL, CRC Press, Chapman & Hall  ---*/
 /*-------------------------------------------------------------------------*/


 /*-------------------------------------------------------*/
 /*---An example of multidimensional scaling, classical---*/
 /*---solution. Example 4.6, p. 207-209                ---*/
 /*-------------------------------------------------------*/


data s;
  input x y;
  datalines;
    0     65
  115     50
  225    120
  175     65
  115    132.5
   30    105
;

 /*------------------------------------------------*/
 /*---This run uses a Euclidean distance matrix ---*/
 /*---The fit is perfect for k=2                ---*/
 /*------------------------------------------------*/
proc iml;
  reset fuzz=1e-11;
  use s; read all var{x y} into s; close s;
  n = nrow(s);
  /* construct distance matrix */
  d1 = J(n,n,.);
  do i = 1 to n;
    d1[i,] = (sqrt((s[i,1] - s[,1])##2 + (s[i,2] - s[,2])##2))`;
  end;
  H = I(n) - J(n,n,1)/n;
  A = -0.5#(d1#d1);
  B1 = H*A*H;
  call eigen(eval,evec,B1);
  print "Eigenvalues of B1", eval;

  /* compute the point configurations in R^2 */
  S2 = evec[,1]#sqrt(eval[1]) || 
       evec[,2]#sqrt(eval[2]);

  /* compute the distance matrices                    */
  d1hat = J(n,n,.);
  do i = 1 to n;
    d1hat[i,] = (sqrt((s2[i,1] - s2[,1])##2 + (s2[i,2] - s2[,2])##2))`;
  end;
  print d1[format=6.2],, d1hat[format=6.2];

  /* apply a Procrustes rotation with scaling of S2 relative to S*/
  X = H*S;
  Y = S2 ;
  Z = Y`*X;
  call svd(v,q,u,z);
  A      = V*U`;
  xbar   = X[:,]`;
  ybar   = Y[:,]`;
  b      = xbar - A`*ybar;
  c      = q[+]/trace(Y*Y`);
  S2star = J(n,2,.);
  do i = 1 to n;
    S2star[i,] = c#((A`*Y[i,]`) + b)` + s[:,];
  end;
  SS = trace(X*X`) + (c*c)*trace(Y*Y`) - 2*c*trace( sqrt(Y`*X*X`*Y) );
  names  = {"x" "y" "s21" "s22" "s2star1" "s2star2"};
  result = s || s2 || s2star;
  create mds1 from result[colname=names];
  append from result;
quit;

title1 'Table 4.5 -- Results for D2';
proc print data=mds1 noobs; 
    format s2: 7.2;
    var x y s21 s22; 
run;

title1 'Table 4.6 -- Results for S2-star';
proc print data=mds1 noobs; 
    format s2: 7.2;
    var x y s2star1 s2star2; 
run;

 /*-----------------------------------------------*/
 /*---This run uses a city-block distance matrix -*/
 /*-----------------------------------------------*/
proc iml;
  reset fuzz=1e-12;
  use s; read all var{x y} into s; close s;
  n = nrow(s);
  /* construct city-block distance matrix */
  d2 = J(n,n,.);
  do i = 1 to n;
    d2[i,] = (abs(s[i,1] - s[,1]) + abs(s[i,2] - s[,2]))`;
  end;
  H  = I(n) - J(n,n,1)/n;
  A  = -0.5#(d2#d2);
  B2 = H*A*H;
  call eigen(eval,evec,B2);
  print "Eigenvalues of B2", eval;
  eval = abs(eval);

  /* compute the point configurations in R^2 */
  S2 = evec[,1]#sqrt(eval[1]) || 
       evec[,2]#sqrt(eval[2]);

  /* compute the distance matrix                    */
  d2hat = J(n,n,.);
  do i = 1 to n;
    d2hat[i,] = (sqrt((s2[i,1] - s2[,1])##2 + (s2[i,2] - s2[,2])##2))`;
  end;
  print d2[format=6.2],, d2hat[format=6.2];

  /* Procrustes rotation with scaling of S2 relative to S*/
  X = S;
  Y = S2;
  Z = Y`*X;
  call svd(v,q,u,z);
  A      = V*U`;
  xbar   = X[:,]`;
  ybar   = Y[:,]`;
  b      = xbar - A`*ybar;
  c      = q[+]/trace(Y*Y`);
  S2star = J(n,2,.);
  do i = 1 to n;
    S2star[i,] = (c#(A`*Y[i,]`) + b)`;
  end;
  SS = trace(H*X*X`*H) + (c*c)*trace(Y*Y`) - 
       2*c*trace( sqrt(Y`*H*X*X`*H*Y) );

  names  = {"x" "y" "s21" "s22" "s2star1" "s2star2"};
  result = s || s2 || s2star;
  create mds2 from result[colname=names];
  append from result;
quit;

title1 'Table 4.5 -- Results for D2';
proc print data=mds2 noobs; 
    format s2: 7.2;
    var x y s21 s22; 
run;

title1 'Table 4.6 -- Results for S2-star';
proc print data=mds2 noobs; 
    format s2: 7.2;
    var x y s2star1 s2star2; 
run;
