
 /*-------------------------------------------------------------------------*/
 /*---Schabenberger, O. and Gotway, C. A. (2005), "Statistical Methods for--*/
 /*---Spatial Data Analysis", Boca Raton, FL, CRC Press, Chapman & Hall  ---*/
 /*-------------------------------------------------------------------------*/

 /*-----------------------------------------------------------*/
 /*---Example 5.8, pages 251-252                           ---*/
 /*---First we produce Table 5.3 with IML, then show how to---*/
 /*---use PROC KRIGE2D and PROC MIXED for kriging          ---*/
 /*-----------------------------------------------------------*/
data fourpoints;
   label x = 'X - coordinate'
         y = 'Y - coordinate'
         z = 'attribute';
   input x y z;
   datalines;
 0  0  5 
 4  0 10 
 0  8 15 
 4  8  6
;

proc iml; 
  reset noprint fw=6;
  use fourpoints; read all var { x y } into s;
                  read all var {  z  } into z;
  close fourpoints;
  n    = Nrow(z);
  ones = J(n,1,1);
  /* calculate Euclidean distances between observed points ||si - sj|| */
  dist_ij= J(n,n,0);
  do i = 1 to n-1;
    dist_ij[i,(i+1):n] = (Sqrt( (s[i,1]-s[i+1:n,1])##2 + (s[i,2]-s[i+1:n,2])##2))`;
  end;
  dist_ij = dist_ij + dist_ij`;

  start semivar;
    dist_0i = (Sqrt( (s0[1]-s[,1])##2 + (s0[2]-s[,2])##2));
    gam    = nugget + (sill-nugget)# ( 1 - exp(-3#dist_ij/8.5));
    gam0   = nugget + (sill-nugget)# ( 1 - exp(-3#dist_0i/8.5));
    gam0st = gam0;
    sigma  = (sill-nugget)#(exp(-3#dist_ij/8.5));
    sigma0 = (sill-nugget)#(exp(-3#dist_0i/8.5));
    sig0st = sigma0;

    /* adjust for nugget effect on the diagonal */
    gam    = (dist_ij > 0)#gam;
    sigma  = (dist_ij> 0)#sigma + sill#(dist_ij < 1E-4);

    /* gamma0 for predicting the Z process */
    do i = 1 to nrow(gam0);
       if dist_0i[i] < 1E-4 then do;
           gam0[i] = 0;
           sigma0[i] = sill;
       end;
    end;
    Igam   = Inv(gam);
    Isigma = Inv(sigma);
    oIgamo = ones`*Igam*ones;
    oIsigo = ones`*ISigma*ones;
    if nugget > 0 then do;  
	   /* assume all of the nugget is measurement error */
       /* gamma0 for predicting the noiseless S process */
        do i = 1 to nrow(gam0st);
           if dist_0i[i] < 1E-4 then do;
              gam0st[i] = nugget;
              sig0st[i] = (sill-nugget);
           end;
        end;
     end; else gam0st = gam0;
  finish semivar;

  start kriging;
    /* 1. this is the minimizer E[{Z(s)-p}^2]    */
    /* 1a) predict in terms of the semivariogram */
    lam   = (gam0 +  ones * (1- ones`*Igam*gam0)/oIgamo)`*Igam;
    lam   = lam#(abs(lam) > 1E-8);
    m     = -(1-ones`*Igam*gam0)/(oIgamo);
    pred  = lam*z;
    /* krvar = gam0`*Igam*gam0 - ((ones`*Igam*gam0 - 1)##2)/(oIgamo); */
    krvar = lam*gam0 + m;
    /* 1b) Use GLS setup and covariances         */
    muhat = (ones`*ISigma*Z)/oISigo;
    GLMPred = muhat + sigma0`*Isigma*(Z-muhat);
    GLMvar  = sill - sigma0`*ISigma*sigma0 + ((1-ones`*ISigma*sigma0)##2)/oIsigo;

    /* 2. this is the minimizer E[{S(s)-p}^2] */
    /* 2a) predict in terms of the semivariogram */
    lamst  = (gam0st +  ones * (1- ones`*Igam*gam0st)/oIgamo)`*Igam;
    predst = lamst*z;
    /* krvarst= gam0`*Igam*gam0 - ((ones`*Igam*gam0 - 1)##2)/(oIgamo); */
    krvarst = lamst*gam0st + m - nugget;
    lamst = lamst#(abs(lamst) > 1E-8);
    /* 2b) Use GLS setup and covariances         */
    GLMPredst = muhat + sig0st`*Isigma*(Z-muhat);
    GLMvarst  = sill - sig0st`*ISigma*sig0st + ((1-ones`*ISigma*sig0st)##2)/oIsigo;
  finish kriging;

  result = J(16,11,0);
  names  = {"Model", "TargX", "TargY", "Sill" , "Nugget", "Pred" , "Variance", "L1","L2","L3","L4"};

  /*--------------------------------------*/
  /*---Target point [2,4]              ---*/
  s0   = { 2 4 };
  sill = 1; nugget = 0;
  run semivar; run kriging;
  result[1,] = 1 || s0[1] || s0[2] || sill || nugget || pred   || krvar   || lam  ;
  result[2,] = 1 || s0[1] || s0[2] || sill || nugget || predst || krvarst || lamst;

  sill = 1; nugget = 0.5;
  run semivar; run kriging;
  result[3,] = 2 || s0[1] || s0[2] || sill || nugget || pred   || krvar   || lam  ;
  result[4,] = 2 || s0[1] || s0[2] || sill || nugget || predst || krvarst || lamst;
  /*--------------------------------------*/

  /*--------------------------------------*/
  /*---Target point [3,2]              ---*/
  s0   = { 3 2 };
  sill = 1; nugget = 0;
  run semivar; run kriging;
  result[5,] = 1 || s0[1] || s0[2] || sill || nugget || pred   || krvar   || lam  ;
  result[6,] = 1 || s0[1] || s0[2] || sill || nugget || predst || krvarst || lamst;

  sill = 1; nugget = 0.5;
  run semivar; run kriging;
  result[7,] = 2 || s0[1] || s0[2] || sill || nugget || pred   || krvar   || lam  ;
  result[8,] = 2 || s0[1] || s0[2] || sill || nugget || predst || krvarst || lamst;
  /*--------------------------------------*/

  /*--------------------------------------*/
  /*---Target point [4,0]              ---*/
  s0   = { 4 0 };
  sill = 1; nugget = 0;
  run semivar; run kriging;
  result[9,]  = 1 || s0[1] || s0[2] || sill || nugget || pred   || krvar   || lam  ;
  result[10,] = 1 || s0[1] || s0[2] || sill || nugget || predst || krvarst || lamst;
  sill = 1; nugget = 0.5;
  run semivar; run kriging;
  result[11,] = 2 || s0[1] || s0[2] || sill || nugget || pred   || krvar   || lam  ;
  result[12,] = 2 || s0[1] || s0[2] || sill || nugget || predst || krvarst || lamst;
  /*--------------------------------------*/

  /*--------------------------------------*/
  /*---Target point [5,-2]             ---*/
  s0   = { 5 -2 };
  sill = 1; nugget = 0;
  run semivar; run kriging;
  result[13,] = 1 || s0[1] || s0[2] || sill || nugget || pred   || krvar   || lam   || SL;
  result[14,] = 1 || s0[1] || s0[2] || sill || nugget || predst || krvarst || lamst || SLst;
  sill = 1; nugget = 0.5;
  run semivar; run kriging;
  result[15,] = 2 || s0[1] || s0[2] || sill || nugget || pred   || krvar   || lam   || SL;
  result[16,] = 2 || s0[1] || s0[2] || sill || nugget || predst || krvarst || lamst || SLst;
  /*---------------------------------------*/

  create krige from result[colname=names];
  append from result;
  close krige;
quit;
proc format; value Model 1='A' 2='B';
data krige; set krige;
  length target $4;
  label Pred   = 'Value'
        TargX  = 'X'
		TargY  = 'Y'
		target = 'Target';
  if (_n_/2 - int(_n_/2)) > 0 then target='Z(s)'; else target='S(s)';
  format Model Model.;
run;
title "Table 5.3, page 252";
proc print data=krige noobs label; 
  format Pred Variance L1-L4 6.2;
  var TargX TargY Model Target Pred Variance L1-L4;
run;


 /*--------------------------------------------------*/
 /*----Below is how to use PROC KRIGE2D to predict---*/
 /*----the Z process                              ---*/
 /*--------------------------------------------------*/
data target;
 input x y z;
 datalines;
2  4 .
3  2 . 
4  0 . 
5 -2 . 
;;
run;

title 'No nugget effect';
proc krige2d data=fourpoints outest=kriged;
  coordinates xcoord=x ycoord=y;
  grid griddata=target xcoord=x ycoord=y;
  predict var=z;
  model form=exponential range=2.8333 scale=1 nugget=0;
run;
data kriged; set kriged; krigvar = stderr**2; run;
proc print data=kriged; run;

title 'With nugget effect';
proc krige2d data=fourpoints outest=kriged;
  coordinates xcoord=x ycoord=y;
  grid griddata=target xcoord=x ycoord=y;
  predict var=z;
  model form=exponential range=2.8333 scale=0.5 nugget=0.5;
run;
data kriged; set kriged; krigvar = stderr**2; run;
proc print data=kriged; run;


 /*--------------------------------------------------*/
 /*--You can also use PROC MIXED to compute kriging -*/
 /*--predictions of the Z process at unobserved locs-*/
 /*--------------------------------------------------*/
data fitthis; set target fourpoints; run;
proc mixed data=fitthis noprofile;
  model z = /outp=p;
  repeated /subject=intercept type=sp(exp)(x y) local;
  parms /* sill   */  0.5
        /* range  */  2.8333
        /* nugget */  0.5 /noiter;
run;
data p; set p; krigvar = stderrpred**2; run;
title 'Kriging Predictions with PROC MIXED';
proc print data=p(where=(z = .)) noobs; 
  var x y z pred krigvar;
run;

