 /*-------------------------------------------------------------------------*/
 /*---Schabenberger, O. and Gotway, C. A. (2005), "Statistical Methods for--*/
 /*---Spatial Data Analysis", Boca Raton, FL, CRC Press, Chapman & Hall  ---*/
 /*-------------------------------------------------------------------------*/


 /*---------------------------------------------------------*/
 /*---Demonstrate effects of semivariogram parameters on ---*/
 /*---kriging weights                                    ---*/
 /*---Target point is at coordinate x=12, y = 30         ---*/
 /*---This program produces Table 5.1, p. 231            ---*/
 /*---------------------------------------------------------*/


 /*---The seven points on page 229                       ---*/
data sevenpoints;
   label x = 'X - coordinate'
         y = 'Y - coordinate'
         z = 'Attribute';
   input n x y z;
   datalines;
1	5	20	100
2	20	2	70
3	25	32	60
4	8	39	90
5	10	17	50
6	35	20	80
7	38	10	40
;


proc iml; reset noprint;
  use sevenpoints; read all var { x y } into s;
                   read all var {  z  } into z;
  close sevenpoints;
  s0   = { 20 20 };
  n    = Nrow(z);
  ones = J(n,1,1);
  /* calculate euclidean distances between observed points ||si - sj|| */
  dist_ij= J(n,n,0);
  do i = 1 to n-1;
    dist_ij[i,(i+1):n] = (Sqrt( (s[i,1]-s[i+1:n,1])##2 + (s[i,2]-s[i+1:n,2])##2))`;
  end;
  dist_ij = dist_ij + dist_ij`;
  /* distances of target from observed points ||s0 - si|| */
  dist_0i = (Sqrt( (s0[1]-s[,1])##2 + (s0[2]-s[,2])##2));

  print s dist_ij dist_0i;

  start semivar;
    /* this is a prediction of the S(s) process in the presence of nugget */
    /* which is assumed to be measurement error.                          */
    if variog = 'exp' then do;
      /* exponential semivariogram model */
      gam    = nugget + (sill-nugget)# ( 1 - exp(-3/range # dist_ij));
      gam0   = nugget + (sill-nugget)# ( 1 - exp(-3/range # dist_0i));
    end;
    if variog = 'gau' then do;
      /* Gaussian semivariogram model */
      gam    = nugget + (sill-nugget)# ( 1 - exp(-3 # (dist_ij/range)##2 ));
      gam0   = nugget + (sill-nugget)# ( 1 - exp(-3 # (dist_0i/range)##2 ));
    end;

    /* adjust for nugget effect on the diagonal */
    gam    = (dist_ij > 0)#gam;
    Igam   = Inv(gam);
    oIgamo = ones`*Igam*ones;
  finish semivar;

  start kriging;
    lam   = (gam0 +  ones * (1- ones`*Igam*gam0)/oIgamo)`*Igam;
    pred  = lam*z;
    krvar = gam0`*Igam*gam0 - ((ones`*Igam*gam0 - 1)##2)/(oIgamo);
    SL    = sqrt(((lam##2)[+] - (lam[+])##2/n)/(n-1));
  finish kriging;

  result = J(6,5+n+1,0);
  names  = {"Range" , "Sill" , "Nugget", "Pred" , "Variance", 
            "L1","L2","L3","L4","L5","L6","L7", "SL"};

  variog = 'exp';
  /* Kriging weights for target */
  range = 20; sill = 10; nugget = 0;
  run semivar; run kriging;
  result[1,] = range || sill || nugget || pred || krvar || lam || SL;

  variog = 'exp';
  /* Kriging weights for target */
  range = 20; sill = 10; nugget = 0;
  run semivar; run kriging;
  result[1,] = range || sill || nugget || pred || krvar || lam || SL;

  range = 10; sill = 10; nugget = 0;
  run semivar; run kriging;
  result[2,] = range || sill || nugget || pred || krvar || lam || SL;

  range = 20; sill = 10; nugget = 5;
  run semivar; run kriging;
  result[3,] = range || sill || nugget || pred || krvar || lam || SL;

  range = 20; sill = 10; nugget = 10;
  run semivar; run kriging;
  result[4,] = range || sill || nugget || pred || krvar || lam || SL;

  range = 20; sill = 20; nugget = 0;
  run semivar; run kriging;
  result[5,] = range || sill || nugget || pred || krvar || lam || SL;

  variog = 'gau';
  range = 20; sill = 10; nugget = 0;
  run semivar; run kriging;
  result[6,] = range || sill || nugget || pred || krvar || lam || SL;

  print result[colname=names format=6.3];

  create krige from result[colname=names];
  append from result;
  close krige;
quit;

proc format; 
   value model 1='A' 2='B' 3='C' 4='D' 5='E';
data krige; set krige; model = _n_;
run;
title1 'Table 5.1 -- page 231';
proc print data=krige noobs; 
   format model model. pred 6.2 variance l1-l7 sl 5.2; 
   var model pred variance L1-L6 sl;
run;

