
 /*-------------------------------------------------------------------------*/
 /*---Schabenberger, O. and Gotway, C. A. (2005), "Statistical Methods for--*/
 /*---Spatial Data Analysis", Boca Raton, FL, CRC Press, Chapman & Hall  ---*/
 /*-------------------------------------------------------------------------*/

%include "&progdir\CNRatioData.sas";

 /*------------------------------------------------------------*/
 /*---Example 6.1 Soil carbon regression. Figure 6.6        ---*/
 /*---Diagnosing a spatial covariance structure with the    ---*/
 /*---approach of Brownie and Gumpertz (1997).              ---*/
 /*------------------------------------------------------------*/

 /*------------------------------------------------------------*/
 /*---Brownie-Gumpertz approach                             ---*/

 /*--- Step 1. Fit the model by REML and get the estimated  ---*/
 /*---         variance matrix (Sigma)                      ---*/
proc mixed data=CNratio;
   model TC = TN / s ddfm=res;
   repeated /subject=intercept type=sp(exp)(x y) r;
   ods output r=Sigma;
   ods output covparms=cp;
   ods exclude r;
run;

 /*--- Step 2. Get the semivariogram of the OLS residuals   ---*/
proc glm data=CNratio;
   model TC = TN;
   output out=olsresid residual=residual;
run;
%let lagd=6;
%let maxlag=44;
proc variogram data=olsresid outvar=olssvar;
  compute lagdistance=&lagd. maxlag=&maxlag.;
  coordinates xcoord=x ycoord=y;
  var residual;
run;


 /*--- Step 3. Compute estimate of the variance of the OLS  ---*/
 /*---         residuals under the assumption that Sigma is ---*/
 /*---         the correct variance matrix                  ---*/
proc iml; 
  use CNratio; read all var{TN}              into x; 
               read all var{x y}             into s;  close CNRatio;
  use sigma;   read all var('col1':'col195') into V;  close sigma;
  use cp;      read all var{estimate}        into cp; close cp;
  n = nrow(x);
  x = J(n,1,1) || x;
  M = I(n) - X*Inv(X`*X)*X`;
  A = M*V*M;
  /* lag class | count | avg. distance | g_e(h) | REML gamma(h) */
  results = J(&maxlag.,4,0);
  do i=1 to &maxlag.; results[i,1] = i; end;
  do i=1 to (n-1);
    do j=(i+1) to n;
      dist  = sqrt((s[i,1]-s[j,1])##2 + (s[i,2]-s[j,2])##2);
	  class = int((dist+(&lagd./2))/&lagd.);
	  if (class <= &maxlag.) then do;
	     svar = 0.5*(A[i,i] + A[j,j]) - A[i,j];
         results[class,2] = results[class,2]+1;
		 results[class,3] = results[class,3]+dist;
		 results[class,4] = results[class,4]+svar;
	  end;
	end; 
  end;
  do i=1 to &maxlag.;
     results[i,3] = results[i,3] / results[i,2]; /* avg. distance */
	 results[i,4] = results[i,4] / results[i,2]; /* g_e(h) */
  end;
  names = {"Lag","Count","distanceG","G"};
  print results[colname=names];
  create gvals from results[colname=names];
  append from results;
run;
quit;
 
goptions vsize=7in hsize=6in;
symbol1 c=black  v=dot    i=none r=1;
symbol2 c=black  v=circle i=none r=1;
axis1 minor=none
      order=(0.0007,0.0010,0.0013,0.0016,0.0019)
	  label=(font='Arial' Height=1.5 angle=90 'Semivariance')
      offset=(0.5in,0.5in)
	  value=(font='Arial' Height=1.3);
axis2 minor=none
      order=(10,70,130,190,250)
	  label=(font='Arial' Height=1.5 'Distance')
      offset=(0.5in,0.5in)
	  value=(font='Arial' Height=1.3);

data plotthis; 
   merge gvals olssvar(where=(distance ne .));
   if distance <= 220;
run;

proc gplot data=plotthis;
  note font='Arial' Height=1.7 move=(2.5,7.3)in 
            'Isotropic Exponential Model';
  plot variog*distance=1
       g*distanceG    =2  / overlay 
                            haxis=axis2 
                            vaxis=axis1 
                            noframe;
run; quit;


 /* ------------------------------------------ */
 /* Use an anisotropic model                   */
 /* ------------------------------------------ */

proc mixed data=CNratio;
   model TC = TN / s;
   repeated /subject=intercept type=sp(expa)(x y) r;
   ods output r=sigma;
   ods output covparms=cp;
   ods exclude r;
run;

proc iml; reset noprint;
  use cnratio; read all var{TN}              into x; 
               read all var{x y}             into s; close cnratio;
  use sigma;   read all var('col1':'col195') into V; close sigma;
  n = nrow(x);
  x = J(n,1,1) || x;
  M = I(n) - X*Inv(X`*X)*X`;
  A = M*V*M;
  /* lag class | count | avg. distance | g_e(h) | REML gamma(h) */
  results = J(&maxlag.,4,0);
  do i=1 to &maxlag.; results[i,1] = i; end;
  do i=1 to (n-1);
    do j=(i+1) to n;
      dist  = sqrt((s[i,1]-s[j,1])##2 + (s[i,2]-s[j,2])##2);
	  class = int((dist+(&lagd./2))/&lagd.);
	  if (class <= &maxlag) then do;
	     svar = 0.5*(A[i,i] + A[j,j]) - A[i,j];
         results[class,2] = results[class,2]+1;
		 results[class,3] = results[class,3]+dist;
		 results[class,4] = results[class,4]+svar;
	  end;
	end; 
  end;
  do i=1 to &maxlag.;
     results[i,3] = results[i,3] / results[i,2]; /* avg. distance */
	 results[i,4] = results[i,4] / results[i,2]; /* g_e(h) */
  end;
  names = {"Lag","Count","distanceG","G"};
  create gvalsan from results[colname=names];
  append from results;
run;
quit;

data plotthis; 
   merge gvalsan 
         olssvar(where=(distance ne .));
   if distance <= 220;
run;
proc gplot data=plotthis;
  note font='Arial' Height=1.7 move=(2.7,7.3)in 
            'Anisotropic Model';
  plot variog*distance=1
       g*distanceG    =2  / overlay 
                            haxis=axis2 
                            vaxis=axis1 
                            noframe;
run; quit;
