 /*-------------------------------------------------------------------------*/
 /*---Schabenberger, O. and Gotway, C. A. (2005), "Statistical Methods for--*/
 /*---Spatial Data Analysis", Boca Raton, FL, CRC Press, Chapman & Hall  ---*/
 /*-------------------------------------------------------------------------*/

 /*---------------------------------------------------------*/
 /*---Example 5.6 -- Trend Surface Model                 ---*/
 /*---------------------------------------------------------*/

 /*---Simulate data set on a 20 x 20 lattice with expo- ---*/
 /*---nential covariance structure                      ---*/
proc sim2d outsim=example_5_6;
   grid x=1 to 20 by 1
        y=1 to 20 by 1;
   simulate numreal=1 form=exponential range=3 nugget=0.25 
            scale=0.5 seed=34585;
run;

 /*---Load various macros                                ---*/
%include "&macrodir\ContiguityWeightsMacro.sas";
%include "&macrodir\MoranIMacro.sas";
%include "&macrodir\MoranResiduals.sas";

%macro trend(p=3);
  proc iml;
    reset noprint;
    use example_5_6; read all var{gxc} into x;
                 read all var{gyc} into y;
    close example_5_6;
    p = &p.;
    n = nrow(x);
	x = x - x[:];
	y = y - y[:];
    x1 = J(n,1,1); 
    y1 = J(n,1,1);
    colcnt = (p+1)*(p+2)/2;
    cnt = 0;
    do k=0 to p; do m=0 to p;
      if (k+m) <= p then do;
        if (k+m = 0) then newmat = J(n,1,1);
        else do;	    
 	      xx = x1; yy=y1;
          do i=1 to k; xx = xx#x; end;
		  if (k=0) then newmat = y1; else newmat=xx;
		  do i=1 to m; newmat = newmat#y; end;
	    end;
	    if (k+m = 0) then xmat = newmat; else xmat = xmat || newmat;
	  end;
    end; end;
    xmat = xmat[,2:ncol(xmat)];
    create xmat from xmat;
    append from xmat;
  quit;
  data fitthis; merge example_5_6(keep=svalue) xmat; run;
  title1 "Trend order surface, order = &p.";
  %let coeffs = %eval((&p+1)*(&p+2)/2 - 1);
  ods listing close;
  proc mixed data=fitthis method=ml;
    model svalue = col1-col&coeffs / outpm=_pm;
    ods output FitStatistics=_fits;
  run;
  ods listing;
  data _fits; 
    set _fits(where=(Descr='-2 Log Likelihood')); 
    aic = value + 2*&coeffs + 2;
	keep aic;
  run;
%mend trend;

%macro onerun(p=1);
  %trend(p=&p);
  %regressI(xmat=xmat,data=example_5_6,z=svalue,weight=weight);
  data _i; merge _ires _fits; p=&p; run;
  proc datasets memtype=data lib=work nolist; 
    append base=_moranI data=_i force;
  run; quit;
%mend onerun;

 /*----------------------------------------------------*/
 /*---Call the macros that fit the trend surface and---*/
 /*---Moran's I for regression residuals in turn for---*/
 /*---the various orders                            ---*/
 /*---Collect the results by appending to a data set---*/
 /*----------------------------------------------------*/

 /* the p=0 case, this will create data set _moranI    */
%ContWght(rows=20,cols=20,move=rook,out=weight);
%MoranI(data=example_5_6,y=svalue,row=gxc,col=gyc,w_data=weight);
ods listing close;
  /* Get -2 Log Likelihood and REML estimate of variance*/
proc mixed data=example_5_6 method=ml;
   model svalue = ;
   ods output FitStatistics=_fits;
proc mixed data=example_5_6 method=reml;
   model svalue = ;
   ods output CovParms =_cp;
run;
ods listing;

data _moranI; 
  merge _moranI(obs=1)
        _fits(where=(Descr='-2 Log Likelihood'))
		_cp(keep=estimate rename=(estimate=s2));
  aic = value + 2;
  p = 0;
  drop assump varI descr value;
run;

%onerun(p= 1);
%onerun(p= 2);
%onerun(p= 3);
%onerun(p= 4);
%onerun(p= 5);
%onerun(p= 6);
%onerun(p= 7);
%onerun(p= 8);
%onerun(p= 9);
%onerun(p=10);
%onerun(p=11);
%onerun(p=12);
%onerun(p=13);
%onerun(p=14);
%onerun(p=15);

title "Table 5.2, page 238";
data _moranI; set _moranI;
  colcnt = (p+1)*(p+2)/2;
proc print data=_moranI noobs label; 
  format s2 7.3 Iobs 7.3 Zobs 7.2 
         Pright pvalue6. AIC 6.1;
  label p      = "Order"
        colcnt = "(p+1)(p+2)/2"
		s2     = "sigma_hat^2"
		Iobs   = "I_res"
		Zobs   = "Z_obs"
		Pright = "p-value"
		aic    = "AIC";
  var p colcnt s2 Iobs Zobs Pright AIC;
run;


 /*--------------------------------------------*/
 /*---Semivariograms for p=0, 5, 6, 7, 8    ---*/
 /*--------------------------------------------*/
proc mixed data=example_5_6;
   model svalue = / outpm=_pm;
run;
proc variogram data=_pm outvar=svar0(where=(rvario ne .));
  compute lagdistance=0.5 maxlag=32 robust;
  coordinates xcoord=gxc ycoord=gyc;
  var resid;
run;

%trend(p=5);
data _pm; merge _pm(keep=resid) example_5_6(keep=gxc gyc);
proc variogram data=_pm outvar=svar5(where=(rvario ne .));
  compute lagdistance=0.5 maxlag=32 robust;
  coordinates xcoord=gxc ycoord=gyc;
  var resid;
run;
%trend(p=6);
data _pm; merge _pm(keep=resid) example_5_6(keep=gxc gyc);
proc variogram data=_pm outvar=svar6(where=(rvario ne .));
  compute lagdistance=0.5 maxlag=32 robust;
  coordinates xcoord=gxc ycoord=gyc;
  var resid;
run;
%trend(p=7);
data _pm; merge _pm(keep=resid) example_5_6(keep=gxc gyc);
proc variogram data=_pm outvar=svar7(where=(rvario ne .));
  compute lagdistance=0.5 maxlag=32 robust;
  coordinates xcoord=gxc ycoord=gyc;
  var resid;
run;
%trend(p=8);
data _pm; merge _pm(keep=resid) example_5_6(keep=gxc gyc);
proc variogram data=_pm outvar=svar8(where=(rvario ne .));
  compute lagdistance=0.5 maxlag=32 robust;
  coordinates xcoord=gxc ycoord=gyc;
  var resid;
run;

data plotthis;
  merge svar0(rename=(rvario=rvario0))
        svar5(rename=(rvario=rvario5)) 
        svar6(rename=(rvario=rvario6))
        svar7(rename=(rvario=rvario7)) 
        svar8(rename=(rvario=rvario8));
run;

goptions hsize=10in vsize=8in;

symbol1 color=black value=none i=join l=1;

axis1 minor=none
      order=(0.3,0.4,0.5,0.6,0.7,0.8)
	  label=(font='Arial' Height=1.5 angle=90 
             'Robust Empirical Semivariogram')
      offset=(0in,0.5in)
	  value=(font='Arial' Height=1.3);
axis2 minor=none
      order=(0,2,4,6,8,10,12,14,16)
	  label=(font='Arial' Height=1.5 'Lag distance')
      offset=(0.5in,0.5in)
	  value=(font='Arial' Height=1.3);
title;
proc gplot data=plotthis;
   note font='Arial' Height=1.3 move=(8,7.0)in 'p=0';
   note font='Arial' Height=1.3 move=(8,2.7)in 'p=5';
   note font='Arial' Height=1.3 move=(8,2.0)in 'p=6';
   note font='Arial' Height=1.3 move=(8,1.5)in 'p=7';
   note font='Arial' Height=1.3 move=(8,1.1)in 'p=8';
   plot rvario0*distance=1 
        rvario5*distance=1
		rvario6*distance=1
		rvario7*distance=1
		rvario8*distance=1
        / overlay noframe 
          vaxis=axis1 haxis=axis2;
run;
quit;


