
 /*-------------------------------------------------------------------------*/
 /*---Schabenberger, O. and Gotway, C. A. (2005), "Statistical Methods for--*/
 /*---Spatial Data Analysis", Boca Raton, FL, CRC Press, Chapman & Hall  ---*/
 /*-------------------------------------------------------------------------*/

 /*-------------------------------------------------------*/
 /*---A spatial CAR model for Poisson data with offset ---*/
 /*---VA blood lead level data                         ---*/
 /*---Results appear in Table 6.5, page 378            ---*/
 /*-------------------------------------------------------*/
 
data VA;
  set SGdata.VA2000;
  logtested72=log(tested72);
run;
data w; set SGdata.vaadjacency; drop id;
run; 

 /*---A Poisson model for independent data with over-  ---*/
 /*---dispersion.                                      ---*/
proc glimmix data=VA;
  model elevated72 = mval1000 pov1000 / 
            link=log dist=poisson 
            offset=logtested72
            solution;
  random _residual_;
  output out=gmxout pred(ilink)=predglm;
run;

/* or use the GENMOD procedure */
/*
proc genmod data=VA;
  model elevated72 = mval1000 pov1000 / 
                  link=log dist=poisson 
                  pscale 
                  offset=logtested72;
run;
*/

proc iml;
  reset noprint fuzz=1e-14;
  use VA; read all var{elevated72}       into y;
          read all var{logtested72}      into offset;
          read all var{mval1000 pov1000} into Xmat; 
  close VA;
  use w; read all into w; close w;
  n     = nrow(xmat);
  Xmat  = J(n,1,1) || Xmat;
  p     = ncol(Xmat);
  e     = eigval(w);
  tol   = 1E-6;
  lbrho = (1/e[><]) + tol;
  ubrho = (1/e[<>]) - tol;
  print "Range of rho parameter",,lbrho ubrho;

  iter    = 1;
  maxiter = 50;
  convcrit= 1E-6;
  betahat = {-2.3 -0.6 0.4}`;
  covparms= { 0 5.7 0.1 }; /* starting value for rho, s2, nugget */
  xres    = covparms;
  blc     = (lbrho // ubrho) ||
            (0     // .    ) ||
            (0     // .    );
  optn    = {0 2};
  conv    = 0;
  rc      = 0;

  /* The function (-2 log restricted pseudo likelihood): */
  /* x[1] = rho, x[2] = s2_1, x[3] = s2_0                */
  start F_PseudoL(x) global(n,p,W,Xmat,muhat,zstar);
    Rmat     = x[2] # Inv(I(n)-x[1]#W) + x[3]#I(n);
    deltavmu = diag(1/sqrt(muhat));
    V        = deltavmu*Rmat*deltavmu;
    VInv     = Inv(V);
    r        = zstar - Xmat*Inv(Xmat`*Vinv*Xmat)*Xmat`*Vinv*Zstar;
    detV     = det(V);
    rVr      = r`*Vinv*r;
    if ((detV <= 0) | (rVr <= 1e-12)) then do;
        f = 1E20;
        print x detV rVr;
    end; else do;
      f = log(detV) + rVr + log(det(Xmat`*Vinv*Xmat)) +
          (n-p)*log(2*arcos(-1));
    end;
    return (f);
  finish;

  /* Derivative of pseudo-log likelihood  */
  /* x[1] = rho, x[2] = s2_1, x[3] = s2_0 */
  start G_PseudoL(x) global(n,p,W,Xmat,muhat,zstar,s2);
     Cvar     = Inv(I(n)-x[1]#W);
     Rmat     = x[2]#Cvar + x[3]#I(n);
     deltavmu = diag(1/sqrt(muhat));
     VInv     = Inv(deltavmu*Rmat*deltavmu);
     C        = (root(Inv(Xmat`*Vinv*Xmat)))`;
     r        = zstar - Xmat*Inv(Xmat`*Vinv*Xmat)*Xmat`*Vinv*Zstar;
     g        = J(1,3,0);
     do i = 1 to 3;
        if (i=1) then do;
		   /* rho */
           Vdotr = x[2] # Cvar*W*Cvar;
        end; else if (i=2) then do;
		   /* s2_1 */
           Vdotr = Cvar;
        end; else do;
		   /* s2_0 */
           Vdotr = I(n);
		end;
        Vdotr = deltavmu*Vdotr*deltavmu;
        g1    = trace(Vinv*Vdotr);
        g2    = -r`*Vinv*Vdotr*Vinv*r;
        g3    = -trace(C`*Xmat`*Vinv*Vdotr*Vinv*Xmat*C);
        g[i]  = g1 + g2 + g3;
     end;
     return (g);
  finish;

  do while ((conv=0) & (iter <= maxiter));
    /*--- Create pseudo-data ---*/
    xbeta = Xmat*betahat;
    muhat = exp(xbeta + offset);
    zstar = (Y-muhat)/muhat + xbeta;
    /*--- Call optimization to estimate covariance parameters---*/
    call nlpqn(rc,xres,"F_PseudoL",covparms,optn,blc) grd = "G_PseudoL";  
    /*--- Update beta ---*/
    Rmat     = xres[2]#Inv(I(n)-xres[1]#W) + xres[3]#I(n);
    deltavmu = diag(1/sqrt(muhat));
    VInv     = Inv(deltavmu*Rmat*deltavmu);
    newbeta  = Inv(Xmat`*Vinv*Xmat)*Xmat`*Vinv*zstar;
    /*--- Check convergence ---*/
    critb = max(abs(betahat - newbeta));
    critc = max(abs(xres    - covparms));
    if ((critc < convcrit) & (critb < convcrit)) then conv=1;
    covparms = xres;
    betahat  = newbeta;
    iter     = iter + 1;
  end;  
  if (conv=1) then do;
     create predvals from muhat[colname="predicted"];
     append from muhat;
     close predvals;
     /*---standard errors for beta hat---*/
     covb = Inv(Xmat`*Vinv*Xmat);
     se   = sqrt(vecdiag(covb));
     result = (betahat || se) //
              (xres[1] || . ) //
              (xres[2] || . ) //
              (xres[3] || . );
     rnames = {"Intercept","mval1000","pov1000","rho","s2_1","s2_0"};
     cnames = {"Estimate","StdErr"};
     reset noname;
     print "--------------- Final Estimates --------------",,
     result[format=best8. rowname=rnames colname=cnames];
     print "----------------------------------------------";
  end;
quit;

libname outdir "&datadir";
data outdir.CARpred; 
  merge VA predvals; 
  mcarnuggetfit    = predicted;
  mcarnuggetfitpct = (predicted/tested72)*100;
run;
