/* DISCLAIMER                                                   
 -----------                                                  
                                                              
 THIS INFORMATION IS PROVIDED "AS IS".
 THERE ARE NO WARRANTIES, EXPRESSED OR IMPLIED, AS TO 
 MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE REGARDING 
 THE ACCURACY OF THE MATERIALS OR CODE CONTAINED HEREIN.   
*/
/* ------------------------------------------------- */
/* --- Author: Oliver Schabenberger                  */
/* --- Macro to calculate Geary's c statistic        */
/*     for attribute variable y= in data set data=   */
/*     Connectivity weights are in data set w_data=  */
/* ------------------------------------------------- */

/* If the data set does not contain row and column   */
/* variables set sort=0                              */

%macro GearyC(data=_last_,row=gxc,col=gyc,y=y,w_data=weights,print=1,sort=1);
  %if &sort=1 %then %do;
    proc sort data=&data out=Gearyc; by &row &col; run;
	%let indata=gearyc;
  %end; %else %do;
    %let indata=&data;
  %end;
  proc iml; reset noprint fw=7;
    use &indata;  read all var {&y} into z;  
                  read all var {&row &col} into locs; close &indata;
    use &w_data;  read all into _W; close &w_data;
    n = nrow(z);
    u = z - z[+]/n;      /* center the observations */
    W = _W / _W[+,+] * n;/* standardize the weights so that 1'W1 = n */
	C = 0;
	do i=1 to n; do j=1 to n;
	  C = C + w[i,j]#((u[i]-u[j])##2); 
	end; end;
    S0 = w[+,+];
    S1 = 0.5 * ((w+w`)##2)[+,+];
    S2 = ((w[+,]` + w[,+])##2)[+];
    b  = n * ((u##4)[+]) / ((u`*u)##2);
	C  = ((n-1)/2)* C / S0 / (u`*u);

    /* Variance of Geary's c under Gaussianity */
    VarC   = (2#S1 + S2)#(n-1) - 4#S0#S0;
	VarC   = VarC / (2#(n+1)#S0#S0);
    StdC   = Sqrt(VarC);
    EC     = 1;
    Zobs   = (C - EC)/StdC;
    pval   = Probnorm(Zobs);
    result = J(2,7,0);
    result[1, ] = 1 || C || EC || VarC || StdC || Zobs || pval;

	/* Variance of Geary's c under randomization */
	VarC = (n-1)#S1#(n##2-3#n+3-(n-1)#b) -
	       0.25#(n-1)#S2#(n##2+3#n-6-(n##2-n+2)#b) +
		   S0#S0*(n##2-3-((n-1)##2)#b);
    VarC = VarC / n/(n-2)/(n-3)/(S0##2);
    StdC = Sqrt(VarC);
    Zobs = (C - EC)/StdC;
    pval = Probnorm(Zobs);
    result[2,] = 2 || C || EC || VarC || StdC || Zobs || pval;

    name = {"Assump","Cobs","EC","VarC","StdC","Zobs","PVal"};
    create _gearyc from result[colname=name]; 
    append from result; close _gearyc;
   quit;
   proc format; 
    value type 2='Randomization' 
	           1='Normality';
   data _gearyc; set _gearyc;
        format Assump type.;
   run;
   %if &print=1 %then %do;
     proc print data=_GearyC label noobs;
        label Assump = 'Method' 
              Cobs = 'Observed C'
              EC   = 'Expected Value'
              StdC = 'Standard Error'
              Zobs = 'Zobs'
              pval = 'Pr(Z < Zobs');
		format CObs EC StdC best7. pval pvalue.;
        var Assump Cobs EC StdC Zobs pval;
      run;
   %end;
%mend Gearyc;

