
 /*-------------------------------------------------------------------------*/
 /*---Schabenberger, O. and Gotway, C. A. (2005), "Statistical Methods for--*/
 /*---Spatial Data Analysis", Boca Raton, FL, CRC Press, Chapman & Hall  ---*/
 /*-------------------------------------------------------------------------*/
 /*
   Macro to perform a Greig-Smith Analysis for a point pattern. 
   Aggregation is performed both vertically and horizontally and the 
   respective mean squares are averaged               

   The group= parameter determines whether the data needs to be grouped 
   (group=1) or whether it is already in the form of quadrat counts 
   (group=0). In the former case the xq= and yq= variable names determine
   the variable names for the quadrats. In the latter case they identify the 
   row and column variable of the quadrats.
 */

/* --- Calculate the mean squares for a set of */
/*     quadrat counts                          */
%macro getms(data = _quads,
             xq   = xq    ,
             yq   = yq    ,
             cnt  = cnt   ,
             q    = 4     ,
             qq   = 8     ,
             n    = 256    );
    %let qqplus = %sysevalf(&qq+1);
    data _blocks; set &data(keep=&xq &yq &cnt);
      retain eps 1E-5;
      array mm{&qq.};
      array bh{&qq.};
      array bv{&qq.};
      do k = (&qq.-1) to 0 by -1;
         m  = 2**k;
         /* Horizontal rectangles */
         dhx = 2**(&q.-1-floor(k/2))+eps;
         dhy = 2**(&q.  -ceil(k/2))+eps;
         bh{k+1} = (2**&q)*floor(&yq/dhy) + floor(&xq/(2*dhx));
         /* Vertical rectangles */
         dvy = 2**(&q.-1-floor(k/2))+eps;
         dvx = 2**(&q.  -ceil(k/2))+eps;
         bv{k+1} = (2**&q)*floor(&xq/dvx) + floor(&yq/(2*dvy));
         mm{k+1} = m;  /* number of pairs */
      end;
      drop eps m;
    run;
    proc means data=_blocks noprint;
      class bv1-bv&qq. bh1-bh&qq.;
      var &cnt;
      output out=_meanout sum=cnt;
      ways 1;
    run;
    proc means data=_blocks noprint;
      var &cnt;
      output out=_total uss=bv&qqplus.;
    run;
    data _meanout; set _meanout;
      array bv{&qq.};
      array bh{&qq.};
      do i = 1 to &qq.;
        bv{i} = cnt*(bv{i} ne .); 
        bh{i} = cnt*(bh{i} ne .);
      end;
    run;

    proc means data=_meanout noprint;
      var bv1-bv&qq. bh1-bh&qq.;
      output out=_Tstats uss=bv1-bv&qq. bh1-bh&qq.;
    run;
    data _Tstats; set _Tstats; if _n_ = 1 then merge _total(keep=bv&qqplus);
      bh&qqplus. = bv&qqplus;
    run;
 
    data _Gvalues; set _Tstats;
      array Tv{&qqplus.} bv1-bv&qqplus.;
      array Th{&qqplus.} bh1-bh&qqplus.;
      do i = 1 to &qqplus.;
        if i = 1 then do;
          Tvert    = Tv{&qqplus.};
          Thoriz   = Th{&qqplus.};
          size     = 1;
       end; else do;
         Tvert    = Tv{i-1};
          Thoriz   = Th{i-1};
          size     = 2**(&qqplus.-i+1);
        end;
        quadrats = 2**(&qq.)/size;
        output;
      end;
      keep size quadrats Tvert  Thoriz ;
    run;
    proc sort data=_Gvalues; by descending size; run;
    data _Gvalues; set _Gvalues;
        Tvlag    = lag(Tvert);
        Thlag    = lag(Thoriz);
        Gv       = 2*Tvert  - Tvlag;
        Gh       = 2*Thoriz - Thlag;
        Gbar     = 0.5*(Gv + Gh);
        Msv      = Gv/(2**(2*&q.));
        Msh      = Gh/(2**(2*&q.));
        Msbar    = 0.5*(Msv + Msh);
        drop Tvlag Thlag;
    run; 
%mend getms;

%macro greigsmith(data  = _data_,
                  xq    = xq    ,
                  yq    = yq    ,
                  cnt   = cnt   ,
                  q     = 4     ,
                  group = 0     ,
                  sims  = 12    ,
                  seed  = 125   );
  options nonotes;
  proc datasets lib=work memtype=data nolist; 
    delete _result; 
  run; quit;

  data _allquads;
    r = 2**(&q.);
    do &xq=1 to r; do &yq = 1 to r; output; end; end; drop r;
  run;
  proc sort data=_allquads; by &xq &yq; run;
 
  %if (&group=1) %then %do;
    %put Grouping the data in &q.*&q. quadrats;
    proc means data=&data noprint;
      var &xq &yq;
      output out=_minmax(drop=_type_ _freq_) min=minx miny max=maxx maxy;
    run;

    data _quads; set &data;
      if _n_ = 1 then merge _minmax;
      r   = 2**(&q.);
      dx  = (maxx-minx)/r;
      dy  = (maxy-miny)/r;
      &xq  = floor((&xq - minx)/dx)+1;
      &yq  = floor((&yq - miny)/dy)+1;
      if &xq > r then &xq=r;
      if &yq > r then &yq=r;
      &cnt = 1;
      drop minx maxx miny maxy;
    run;
    proc sort data=_quads; by &xq &yq; run;
    proc means data=_quads noprint; by &xq &yq;
       var &cnt; output out=_quadcnts(drop=_type_ _freq_) sum=cnt;
    run;
    data _quads; merge _allquads _quadcnts; 
      by &xq &yq; 
      if &cnt = . then &cnt = 0;
      loc = _n_;
    run;
  %end; %else %do;
    proc sort data=&data; by &xq &yq; run;

    data _quads; merge _allquads &data; 
      by &xq &yq; 
      if &cnt = . then &cnt = 0;
      loc = _n_;
    run;
  %end;

  %let qq = %sysevalf(2*&q);
  %let n = %sysevalf(2**(2*&q.)); /* number of quadrats */

  %if (&sims=0) %then %do; /* no simulations required */
    %getms(data=_quads,xq=&xq,yq=&yq,cnt=&cnt,q=&q,qq=&qq,n=&n);
    proc sort data=_gvalues; by size; run;
    data _result; set _gvalues; 
      label size    ='block size'
            quadrats='No. of blocks'
            Tvert   ='SS(vertical)'
            Thoriz  ='SS(horiz)'
            MSv     ='MS(vertical)'
            MSh     ='MS(horiz)'
            MSbar   ='AverageMS'
            lower95 ='Lower 95% bound'
            upper95 ='Upper 95% bound';
       if (Msbar ne .) then do;
         lower95 = lag(MSbar)/(quadrats/2)*Cinv(0.025,(quadrats/2));
         upper95 = lag(Msbar)/(quadrats/2)*Cinv(0.975,(quadrats/2));
       end;
    run;
    title1 "Greig-Smith Analysis of Quadrat Counts for Data Set &data..";
    title2 "2^&q. * 2^&q. quadrats. No randomization inference";
    proc print data=_result label;
     var Tvert Thoriz Msv Msh Msbar lower95 upper95;
     id  size quadrats;
    run;
  %end; %else %do; /* Monte Carlo analysis */
    /* generate &sims random arrangements of the quadrat counts */
    /* include the observed arrangement as the first set        */
    data _sims;
      array arr{&sims.} _s1-_s&sims.;
      do i=1 to &n;
        do j = 1 to &sims.; arr{j} = ranuni(&seed); end;
        output;
      end;
      drop i j;
    run;
    proc rank data=_sims out=_ranks ties=high;
      var _s1-_s&sims.; ranks _r1-_r&sims.; 
    run;
    proc transpose data=_ranks(keep=_r1-_r&sims.) out=_tranks prefix=_r; run;
    proc transpose data=_quads(keep=&cnt) out=_tquads; run;
    data _sims; set _tranks; if _n_ = 1 then set _tquads;
      array obs{&n} col1-col&n;
      array ss{&n}  _r1-_r&n;
      array out{&n} cnt1-cnt&n;
      if _n_ = 1 then do;
         do i = 1 to &n; out{i} = obs{i}; end;
         sim = 0;
         output;
         do i = 1 to &n; out{i} = obs{ss{i}}; end;
         sim = _n_;
         output;
      end; else do;
         do i = 1 to &n; out{i} = obs{ss{i}}; end;
         sim = _n_;
         output;
      end;
      keep cnt1-cnt&n. sim;
    run;
    proc transpose data=_sims out=_analyzesims prefix=_s; run;   
    /* combine with the row/column identifiers in original data */
    data _analyzesims; merge _quads(keep=&xq &yq) _analyzesims; 
      if (&xq ne .);
    run; 

    %do s=1 %to %eval(&sims.+1);
      %put %str(      ) Running simulation %eval(&s-1) of &sims..;
      %getms(data=_analyzesims,xq=&xq,yq=&yq,cnt=_s&s,q=&q,qq=&qq,n=&n);
      data _gvalues; set _gvalues; sim=&s.-1; run;
      proc datasets lib=work memtype=data nolist; 
         append base=_result data=_gvalues force;
      run; quit;
    %end;
    proc sort data=_result(where=(sim > 0)) out=_sims; by size; run;
    /* average the simulations */
    proc univariate data=_sims noprint;
      by size;
      var msv msh msbar;
      output out=_simmeans min=msvmin  mshmin msbarmin
                           max=msvmax  mshmax msbarmax
                           mean=msvmn  mshmn  msbarmn
                           pctlpre=msv msh msbar pctlpts=2.5
                           pctlpre=msv msh msbar pctlpts=97.5;
      id quadrats;
    run;
    proc sort data=_result(where=(sim=0)) out=_result; by size; run;
    data _result; merge _result _simmeans; by size; 
      label size    ='block size'
            quadrats='No. of blocks'
            Tvert   ='SS(vertical)'
            Thoriz  ='SS(horiz)'
            MSv     ='MS(vertical)'
            MSh     ='MS(horiz)'
            MSbar   ='AverageMS'
			MSbarMin='Min(AverageMS)'
			MSbarMax='Max(AverageMS)'
            lower95 ='Lower 95% bound'
            upper95 ='Upper 95% bound';
       if (Msbar ne .) then do;
         lower95 = lag(MSbar)/(quadrats/2)*Cinv(0.025,(quadrats/2));
         upper95 = lag(Msbar)/(quadrats/2)*Cinv(0.975,(quadrats/2));
       end;
    run;
    title1 "Greig-Smith Analysis of Quadrat Counts for Data Set &data..";
    title2 "2^&q. * 2^&q. quadrats, &sims. random permutations, seed=&seed..";
	options nodate nonumber;
    proc print data=_result label; 
	   format Ms: 6.2;
       var Tvert Thoriz Msv Msh Msbar MSbarMin MsbarMax;
       id size quadrats;
    run;
    proc print data=_result label; 
       var lower95 upper95 Msv2_5 Msv97_5 Msh2_5 Msh97_5 Msbar2_5 Msbar97_5;
       id size quadrats;
    run;
    symbol1 i=join l=31 v=circle color=red;
    symbol2 i=join l=1           color=black w=2;
/*
	proc gplot data=_result(where=(msbar ne .));
       plot msbar*size=1 msbar2_5*size=2 msbar97_5*size=2 / overlay;
    run; quit;
*/
  %end; /* end of %if (&sims=0) */

  proc datasets lib=work memtype=data nolist;
    delete _Tstats _blocks _meanout _allquads _quads _quadcnts _minmax
           _tquads _ranks _tranks _sims _analyzesims _gvalues _simmeans;
  run; quit;

  options notes;
%mend greigsmith;


