/* DISCLAIMER                                                   
 -----------                                                  
                                                              
 THIS INFORMATION IS PROVIDED "AS IS".
 THERE ARE NO WARRANTIES, EXPRESSED OR IMPLIED, AS TO 
 MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE REGARDING 
 THE ACCURACY OF THE MATERIALS OR CODE CONTAINED HEREIN.   
*/
/* ------------------------------------------------- */
/* --- Author: Oliver Schabenberger                  */
/* --- Macro to calculate Moran's global I statistic */
/*     for attribute variable y= in data set data=   */
/*     Connectivity weights are in data set w_data=  */
/* ------------------------------------------------- */

/* If the data set does not contain row and column   */
/* variables set sort=0                              */
/* If local=1 then local Moran's I statistics are    */
/* calculated.                                       */

%macro MoranI(data=_last_,row=gxc,col=gyc,y=y,w_data=weights,print=1,sort=1,local=0);
  %if &sort=1 %then %do;
    proc sort data=&data out=moranI; by &row &col; run;
	%let indata=moranI;
  %end; %else %do;
    %let indata=&data;
  %end;
  proc iml; reset noprint fw=7;
    use &indata;  read all var {&y} into z;  
                  read all var {&row &col} into locs; close &indata;
    use &w_data;  read all into C; close &w_data;
    n = nrow(z);
    u = z - z[+]/n;      /* center the observations */
    W = C / c[+,+] * n;  /* standardize the weights so that 1'W1 = n  and I = (u'Wu)/(u'u) */
    I  = (u`*W*u)/(u`*u);
    result=J(2,7,0);
    EI = - 1/ (n-1);
    S0 = w[+,+];
    S1 = 0.5 * ((w+w`)##2)[+,+];
    S2 = ((w[+,]` + w[,+])##2)[+];
    b  = n * ((u##4)[+]) / ((u`*u)##2);

    /* Variance of Moran's I under randomization */
    VarI   = n * ((n##2 -3*n + 3)*S1 - n*S2 + 3*(S0##2)) - b*((n##2-n)*S1 - 2*n*S2 + 6*(S0##2));
    VarI   = VarI / (n-1)/(n-2)/(n-3)/(S0##2);
    VarI   = VarI - 1/((n-1)##2);
    StdI   = Sqrt(VarI);
    Zobs   = (I - EI)/StdI;
    pright = 1-Probnorm(Zobs);
    result[2,] = 2 || i || Ei || VarI || StdI || Zobs || pright;

    /* Variance of Moran's I under Gaussianity */
    VarI = (n*n*S1 - n*S2 + 3*S0##2)/(S0##2 * (n*n - 1));
    VarI = VarI - 1/((n-1)##2);
    StdI = Sqrt(VarI);
    Zobs   = (I - EI)/StdI;
    pright = 1-Probnorm(Zobs);
    result[1, ] = 1 || i || Ei || VarI || StdI || Zobs || pright;
    name = {"Assump","Iobs","EI","VarI","StdI","Zobs","PRight"};
    create _moranI from result[colname=name]; append from result; close _MoranI;

    /* Is the local version of Moran's I desired ? */
    %if &local=1 %then %do;
      LocalI = J(n,4,.);
      sumui2 = (u`*u);
      do i = 1 to n;
        locMoran = (n*u[i])#(W[i,]*u);
        Locali[i,1] = Locs[i,1];
        Locali[i,2] = Locs[i,2];
        LocalI[i,3] = locMoran/Sumui2;
        LocalI[i,4] = (-(W[i,][+])/(n-1));
      end;
      name = {"&row","&col","LocalI","ELocalI"};
      create _localI from Locali[colname=name]; append from Locali; close _localI;
    %end;

   quit;
   proc format; 
    value type 2='Randomization' 
	           1='Normality';
   data _moranI; set _moranI; 
        format Assump type.;
   run;
   %if &print=1 %then %do;
     proc print data=_MoranI label noobs;
        label Assump = 'Method' 
              Iobs = 'Observed I'
              EI   = 'Expected Value'
              StdI = 'Standard Error'
              Zobs = 'Zobs'
              Pright = 'Pr(Z > Zobs');
		format IObs EI STDI best7. pright pvalue.;
        var Assump Iobs EI StdI Pright;
      run;
      %if &local=1 %then %do;
        title 'First observations of local I (WORK._LOCALI)';
        proc print data=_localI(obs=50); run;
      %end;
   %end;
%mend MoranI;

