/* DISCLAIMER                                                   
 -----------                                                  
                                                              
 THIS INFORMATION IS PROVIDED "AS IS".
 THERE ARE NO WARRANTIES, EXPRESSED OR IMPLIED, AS TO 
 MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE REGARDING 
 THE ACCURACY OF THE MATERIALS OR CODE CONTAINED HEREIN.   
*/

/* ------------------------------------------------------ */
/* --- %RegressI                                          */
/*     Calculate Moran's I from regression residual in a  */
/*     linear model                                       */
/* ------------------------------------------------------ */

/* --- Macro Arguments
where       xmat   =   a data set containing the regressor variables apart from the intercept
            data   =   a data set containing the response variable
            z      =   the response variable Zs in data set data=
            weight =   a data set containing the weight matrix W
*/

%macro RegressI(xmat=,data=_last_,z=z,weight=w_data);
proc iml; reset noprint fw=6;
  use &xmat;   read all into X; close &xmat;
  use &data;   read all var {&z } into z; close &data;
  use &weight; read all into W; close &weight;
  n    = nrow(z);
  X    = J(n,1,1) || X;
  XpXI = Inv(X`*X);
  b    = XpXi*X`*z;                /* Regression coefficients   */
  k    = ncol(X)-1;
  H    = X*Inv(X`*X)*X`;           /* Hat matrix                */
  M    = I(n) - H;                 /* Error projector           */
  s2   = (z`*M*z)/(n-ncol(x));     /* Residual sum of square    */
  seb  = sqrt(vecdiag(s2*XpXi));   /* ese of regression coeffs. */
  e    = M*z;                      /* Residuals from regression */
  AOV  = J(3,6,.);                 /* OLS Analysis of Variance  */
  AOV[2,1] = z`*M*z;; AOV[2,2] = n-ncol(X);
  AOV[3,1] = z`*z - n*(z[:]**2); AOV[3,2] = n-1;
  AOV[1,1] = AOV[3,1]-AOV[2,1];  AOV[1,2] = k;
  AOV[1,3] = AOV[1,1]/AOV[1,2];
  AOV[1,6] = AOV[1,1]/AOV[3,1];
  AOV[2,3] = AOV[2,1]/AOV[2,2];
  AOV[1,4] = AOV[1,3]/AOV[2,3];
  AOV[1,5] = 1-ProbF(AOV[1,4],AOV[1,2],AOV[2,2]);
  cnama    = {"SS","df","MS","F","Pr>F","R2"};
  rnama    = {"Model","Error","C.Total"};
  mattrib AOV rowname=rnama colname=cnama label='OLS Analysis of Variance' format=best7.;
  tobs = b/seb;
  pval = 2*(1-ProbT(abs(tobs),n-ncol(x)));
  ols  = b || seb || tobs || pval;
  cnam = {"Estimate", "StdErr", "Tobs", "Pr>|T|"};
  rnam = {"Intcpt"} // contents(&xmat);
  mattrib ols rowname=rnam colname=cnam label='OLS Estimates' format=best7.;

  /* Calculate the global Moran's I from the residuals */
  S0      = W[+,+];
  A       = XpXi*X`*W*X;
  MoranI  = (n/S0)*(e`*W*e)/(e`*e);
  EMoranI = -n/(n-k)*trace(A)/S0;
  U  = (W+W`);
  B  = XpXI*X`*(U*U)*X;
  S1 = 0.5 * ((w+w`)##2)[+,+];
  VmoranI = n*n/(S0##2)/(n-k)/(n-k+2) * ( S1 + 2*trace(A*A) - trace(B) - 2*(trace(A)##2)/(n-k) );
  SMoranI = Sqrt(VMoranI);
  ZMoranI = (MoranI-EMoranI)/Sqrt(VMoranI);
  PValI   = 1-ProbNorm(abs(ZMoranI));
  print "Ordinary Least Squares Regression Results", AOV , ols;
  I       = MoranI || EMoranI || SMoranI || ZMoranI || PvalI;
  cnamI   = {"I*", "E[I*]","SE[I*]","Zobs","Pr > Zobs"};
  mattrib I colname=cnamI format=best7. label="Global Moran's I";
  print I;
  names = {"s2", "Iobs", "EI","StdI","Zobs","Pright"};
  I = s2 || I;
  create _ires from I[colname=names];
  append from I;
  close _ires;
quit;
%mend RegressI;
